from typing import Union

from . import barrier, bec, bragg, paint_1d, transistor

BertJobSchema = Union[
    bec.BecCreateJob,
    bec.BecCompleteJob,
    bec.BecCreateBatchJob,
    bec.BecCompleteBatchJob,
    bec.BecCreateJobResponse,
    bec.BecCompleteJobResponse,
    bec.BecCreateBatchJobResponse,
    bec.BecCompleteBatchJobResponse,
    barrier.BarrierCreateJob,
    barrier.BarrierCompleteJob,
    barrier.BarrierCreateBatchJob,
    barrier.BarrierCompleteBatchJob,
    barrier.BarrierCreateJobResponse,
    barrier.BarrierCompleteJobResponse,
    barrier.BarrierCreateBatchJobResponse,
    barrier.BarrierCompleteBatchJobResponse,
    transistor.TransistorCreateJob,
    transistor.TransistorCompleteJob,
    transistor.TransistorCreateBatchJob,
    transistor.TransistorCompleteBatchJob,
    transistor.TransistorCreateJobResponse,
    transistor.TransistorCompleteJobResponse,
    transistor.TransistorCreateBatchJobResponse,
    transistor.TransistorCompleteBatchJobResponse,
    bragg.BraggCreateJob,
    bragg.BraggCompleteJob,
    bragg.BraggCreateBatchJob,
    bragg.BraggCompleteBatchJob,
    bragg.BraggCreateJobResponse,
    bragg.BraggCompleteJobResponse,
    bragg.BraggCreateBatchJobResponse,
    bragg.BraggCompleteBatchJobResponse,
    paint_1d.Paint1DCreateJob,
    paint_1d.Paint1DCompleteJob,
    paint_1d.Paint1DCreateBatchJob,
    paint_1d.Paint1DCompleteBatchJob,
    paint_1d.Paint1DCreateJobResponse,
    paint_1d.Paint1DCompleteJobResponse,
    paint_1d.Paint1DCreateBatchJobResponse,
    paint_1d.Paint1DCompleteBatchJobResponse,
]
